@@ -39,17 +39,14 @@ sub new {
 
     $buffer = '' unless defined $buffer;
 
-    if (Encode::is_utf8($buffer)) {
-        $self->{buffer} = Encode::encode('UTF-8', $buffer);
-    }
-    else {
-        $self->{buffer} = $buffer;
-    }
-
     if (defined($self->{type}) && defined($TYPES{$self->{type}})) {
         $self->opcode($TYPES{$self->{type}});
     }
 
+    $self->{buffer} = $self->is_text ?
+        Encode::encode('UTF-8', $buffer) :
+        $buffer;
+
     $self->{version} ||= 'draft-ietf-hybi-17';
 
     $self->{fragments} = [];
@@ -361,8 +358,10 @@ L<Math::Random::Secure> is installed it is used instead.
     Protocol::WebSocket::Frame->new('data');   # same as (buffer => 'data')
     Protocol::WebSocket::Frame->new(buffer => 'data', type => 'close');
 
-Create a new L<Protocol::WebSocket::Frame> instance. Automatically detect if the
-passed data is a Perl string (UTF-8 flag) or bytes.
+Create a new L<Protocol::WebSocket::Frame> instance. When creating a C<text>
+frame (the default behaviour if C<type> is not specified) the string data is
+encoded into UTF-8 bytes; for other frame types it is assumed to already be in
+bytes.
 
 When called with more than one arguments, it takes the following named arguments
 (all of them are optional).
@@ -433,6 +432,13 @@ Check if frame is of close type.
 
 Get/set opcode of the frame.
 
+=head2 C<masked>
+
+    $masked = $frame->masked;
+    $frame->masked(1);
+
+Get/set masking of the frame.
+
 =head2 C<append>
 
     $frame->append($chunk);
